// -*- C++ -*-
/*!
 * @file  SavedImage.cpp
 * @brief SavedImage component
 * @date $Date$
 *
 * $Id$
 */

#include "SavedImage.h"

// Module specification
// <rtc-template block="module_spec">
static const char* savedimage_spec[] =
  {
    "implementation_id", "SavedImage",
    "type_name",         "SavedImage",
    "description",       "SavedImage component",
    "version",           "1.0.0",
    "vendor",            "Shintaro",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.flipMode", "0",

    // Widget
    "conf.__widget__.flipMode", "radio",
    // Constraints
    "conf.__constraints__.flipMode", "(0,-1,1)",

    "conf.__type__.flipMode", "int",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
SavedImage::SavedImage(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_originalImageIn("originalImage", m_originalImage),
    m_ShotTimingIn("ShotTiming", m_ShotTiming),
    m_flippedImageOut("flippedImage", m_flippedImage),
    m_savedokOut("savedok", m_savedok)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
SavedImage::~SavedImage()
{
}



RTC::ReturnCode_t SavedImage::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("originalImage", m_originalImageIn);
  addInPort("ShotTiming", m_ShotTimingIn);
  
  // Set OutPort buffer
  addOutPort("flippedImage", m_flippedImageOut);
  addOutPort("savedok", m_savedokOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("flipMode", m_flipMode, "0");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t SavedImage::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SavedImage::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SavedImage::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t SavedImage::onActivated(RTC::UniqueId ec_id)
{
	saved_image_counter = 1;

	// OutPort̉ʃTCY̏
	m_flippedImage.width = 0;
	m_flippedImage.height = 0;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t SavedImage::onDeactivated(RTC::UniqueId ec_id)
{
	if (!m_imageBuff.empty())
	{
		cv::destroyAllWindows();
		// 摜p̉
		m_imageBuff.release();
		m_flipImageBuff.release();
	}
  return RTC::RTC_OK;
}


RTC::ReturnCode_t SavedImage::onExecute(RTC::UniqueId ec_id)
{
	// Vf[^̃`FbN
	if (m_originalImageIn.isNew()) {
		// InPortf[^̓ǂݍ
		m_originalImageIn.read();

		// InPortOutPort̉ʃTCYуC[Wp̊m
		if (m_originalImage.width != m_flippedImage.width || m_originalImage.height != m_flippedImage.height)
		{
			m_flippedImage.width = m_originalImage.width;
			m_flippedImage.height = m_originalImage.height;

			m_imageBuff.create(cv::Size(m_originalImage.width, m_originalImage.height), CV_8UC3);
			m_flipImageBuff.create(cv::Size(m_originalImage.width, m_originalImage.height), CV_8UC3);


		}

		// InPort̉摜f[^m_imageBuffɃRs[
		memcpy(m_imageBuff.data, (void *)&(m_originalImage.pixels[0]), m_originalImage.pixels.length());

		// InPort̉摜f[^𔽓]B m_flipMode 0: X, 1: Y, -1: ̎
		cv::flip(m_imageBuff, m_flipImageBuff, m_flipMode);

		// 摜f[^̃TCY擾
		int len = m_flipImageBuff.channels() * m_flipImageBuff.cols * m_flipImageBuff.rows;
		m_flippedImage.pixels.length(len);

		//摜\

		cv::imshow("test1", m_imageBuff);
		cv::waitKey(30);

		// 摜f[^ۑ
		if (m_ShotTimingIn.isNew())
		{
			m_ShotTimingIn.read();

			if (m_ShotTiming.data == 1) {

				char test1[100];
				sprintf(test1, "test1%03d.png", ++saved_image_counter);

				cv::imwrite(test1, m_imageBuff);
				m_savedok.data = 1;
				m_savedokOut.write();
			}
		}
	
		// ]摜f[^OutPortɃRs[

		memcpy((void *)&(m_flippedImage.pixels[0]), m_flipImageBuff.data, len);

		// ]摜f[^OutPorto͂B
		m_flippedImageOut.write();
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t SavedImage::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SavedImage::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SavedImage::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SavedImage::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SavedImage::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void SavedImageInit(RTC::Manager* manager)
  {
    coil::Properties profile(savedimage_spec);
    manager->registerFactory(profile,
                             RTC::Create<SavedImage>,
                             RTC::Delete<SavedImage>);
  }
  
};


